(*
 * copyright (c) 2006 Mans Rullgard
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

   ===

   revision 15120, 31-aug-2008

   delphi conversion of libavcodec/avio.h

   slightly.fat.hamster@gmail.com, originally by Victor Zinetz

   24-sept-2008

   ===

 *)

(* output byte stream handling *)
unit avio;

interface

uses
  windows;

const
  LIBAVFORMAT_VERSION_MAJOR = 52;
  LIBAVFORMAT_VERSION_MINOR = 22;
  LIBAVFORMAT_VERSION_MICRO = 1;

  LIBAVFORMAT_VERSION_INT = LIBAVFORMAT_VERSION_MAJOR shl 16 +
                            LIBAVFORMAT_VERSION_MINOR shl 8 +
                            LIBAVFORMAT_VERSION_MICRO;

  dll_name = 'avformat-52.dll';

  URL_RDONLY = 0;
  URL_WRONLY = 1;
  URL_RDWR   = 2;

(**
 * Passing this as the "whence" parameter to a seek function causes it to
 * return the filesize without seeking anywhere. Supporting this is optional.
 * If it is not supported then the seek function will return <0.
 *)
  AVSEEK_SIZE = $10000;

  URL_EOF = -1;

type
  offset_t = int64;
  int = integer;

//* unbuffered I/O *)

(*
 * URL Context.
 * New fields can be added to the end with minor version bumps.
 * Removal, reordering and changes to existing fields require a major
 * version bump.
 * sizeof(URLContext) must not be used outside libav*.
 *)

  PURLContext = ^TURLContext;
  PURLProtocol = ^TURLProtocol;
  TURLContext = record
  {$if LIBAVFORMAT_VERSION_MAJOR >= 53}
    av_class: PAVClass; ///< information for av_log(). Set by url_open().
  {$ifend}

    prot: PURLProtocol;
    flags: int;
    is_streamed: int;  (**< true if streamed (no seek possible), default = false *)
    max_packet_size: int;  (**< if non zero, the stream is packetized with this max packet size *)
    priv_data: pointer;
    filename: pchar; (**< specified filename *)
  end;

  PURLPollEntry = ^TURLPollEntry;
  TURLPollEntry = record {sizeof = 12}
    handle: PURLContext;
    events: int;
    revents: int;
  end;

  TURLProtocol = record {sizeof = 36}
    name: pchar;
    url_open: function (h: PURLContext; const filename: pchar; flags: int): int; cdecl;
    url_read: function (h: PURLContext; buf: pchar; size: int): int; cdecl;
    url_write: function (h: PURLContext; buf: pchar; size: int): int; cdecl;
    url_seek: function (h: PURLContext; pos: offset_t; whence: int): offset_t; cdecl;
    url_close: function (h: PURLContext): int; cdecl;
    next: PURLProtocol;
    url_read_pause: function (h: PURLContext; pause: int): int; cdecl;
    url_read_seek: function (h: PURLContext; stream_index: int; timestamp: int64; flags: int): offset_t; cdecl;
  end;

//extern URLProtocol *first_protocol;
//extern URLInterruptCB *url_interrupt_cb;

(**
 * Bytestream IO Context.
 * New fields can be added to the end with minor version bumps.
 * Removal, reordering and changes to existing fields require a major
 * version bump.
 * sizeof(ByteIOContext) must not be used outside libav*.
 *)
  PByteIOContext = ^TByteIOContext;
  TByteIOContext = record { sizeof = 88 }
    buffer: pchar;
    buffer_size: int;
    buf_ptr, buf_end: pchar;
    opaque: pointer;
    read_packet: function (opaque: pointer; buf: PByte; buf_size: int): int; cdecl;
    write_packet: function (opaque: pointer; buf: PByte; buf_size: int): int; cdecl;
    seek: function (opaque: pointer; offset: offset_t; whence: int): offset_t; cdecl;
    pos: offset_t; (**< position in the file of the current buffer *)
    must_flush: int; (**< true if the next seek should flush *)
    eof_reached: int; (**< true if eof reached *)
    write_flag: int;  (**< true if open for writing *)
    is_streamed: int;
    max_packet_size: int;
    checksum: cardinal;
    checksum_ptr: pchar;
    update_checksum: function (checksum: cardinal; const buf: PByte; size: cardinal): cardinal; cdecl;
    error: int;         ///< contains the error code or 0 if no error happened
    read_pause: function (opaque: pointer; pause: int): int; cdecl;
    read_seek: function (opaque: pointer; stream_index: int; timestamp: int64; flags: int): offset_t; cdecl;
  end;


  function av_protocol_next (p: PURLProtocol): PURLProtocol;
    cdecl; external dll_name;

  function register_protocol (protocol: PURLProtocol): int;
    cdecl; external dll_name;

(**
 * Seek to a given timestamp relative to some component stream.
 * Only meaningful if using a network streaming protocol (e.g. MMS.).
 * @param stream_index The stream index that the timestamp is relative to.
 *        If stream_index is (-1) the timestamp should be in AV_TIME_BASE
 *        units from the beginning of the presentation.
 *        If a stream_index >= 0 is used and the protocol does not support
 *        seeking based on component streams, the call will fail with ENOTSUP.
 * @param timestamp timestamp in AVStream.time_base units
 *        or if there is no stream specified then in AV_TIME_BASE units.
 * @param flags Optional combination of AVSEEK_FLAG_BACKWARD, AVSEEK_FLAG_BYTE
 *        and AVSEEK_FLAG_ANY. The protocol may silently ignore
 *        AVSEEK_FLAG_BACKWARD and AVSEEK_FLAG_ANY, but AVSEEK_FLAG_BYTE will
 *        fail with ENOTSUP if used and not supported.
 * @return >= 0 on success
 * @see AVInputFormat::read_seek
 *)
  function av_url_read_seek (h: PURLContext; stream_index: int; timestamp: int64; flags: int): offset_t;
    cdecl; external dll_name;

(**
 * Return the maximum packet size associated to packetized file
 * handle. If the file is not packetized (stream like HTTP or file on
 * disk), then 0 is returned.
 *
 * @param h file handle
 * @return maximum packet size in bytes
 *)
  function url_get_max_packet_size (h: PURLContext): int;
    cdecl; external dll_name;
  procedure url_get_filename (h: PURLContext; buf: Pchar; buf_size: int);
    cdecl; external dll_name;

(**
 * The callback is called in blocking functions to test regulary if
 * asynchronous interruption is needed. AVERROR(EINTR) is returned
 * in this case by the interrupted function. 'NULL' means no interrupt
 * callback is given.
 *)
  procedure url_set_interrupt_cb (interrupt_cb: pointer {PURLInterruptCB});
    cdecl; external dll_name;
(* not implemented *)
  function url_poll (poll_table: PURLPollEntry; n: int; timeout: int): int;
    cdecl; external dll_name;
(**
 * Pause and resume playing - only meaningful if using a network streaming
 * protocol (e.g. MMS).
 * @param pause 1 for pause, 0 for resume
 *)
  function av_url_read_pause (h: PURLContext; pause: int): int;
    cdecl; external dll_name;

  function url_open_protocol (var puc: PURLContext; up: PURLProtocol; const filename: pchar; flags: int): int;
    cdecl; external dll_name;
  function url_open (var h: PURLContext; const filename: pchar; flags: int): int;
    cdecl; external dll_name;
  function url_read (h: PURLContext; buf: pchar; size: int): int;
    cdecl; external dll_name;
  function url_write (h: PURLContext; buf: pchar; size: int): int;
    cdecl; external dll_name;
  function url_seek (h: PURLContext; pos: offset_t; whence: int): offset_t;
    cdecl; external dll_name;
  function url_close (h: PURLContext): int;
    cdecl; external dll_name;
  function url_exist (const filename: pchar): int;
    cdecl; external dll_name;
  function url_filesize (h: PURLContext): offset_t;
    cdecl; external dll_name;

  function init_put_byte (s: PByteIOContext; buffer: pchar; buffer_size: int;
                  write_flag: int; opaque: pointer;
                  read_packet: pointer; //int (*read_packet)(void *opaque, uint8_t *buf, int buf_size),
                  write_packet: pointer; //int (*write_packet)(void *opaque, uint8_t *buf, int buf_size),
                  seek: pointer //offset_t (*seek)(void *opaque, offset_t offset, int whence)
                  ): int;
    cdecl; external dll_name;

  function av_alloc_put_byte(buffer: pchar;
                  buffer_size: int;
                  write_flag: int;
                  opaque: pointer;
                  read_packet: pointer; //int (*read_packet)(void *opaque, uint8_t *buf, int buf_size),
                  write_packet: pointer; //int (*write_packet)(void *opaque, uint8_t *buf, int buf_size),
                  seek: pointer //offset_t (*seek)(void *opaque, offset_t offset, int whence));
                  ): PByteIOContext;
    cdecl; external dll_name;

  procedure put_byte (s: PByteIOContext; b: int);
    cdecl; external dll_name;
  procedure put_buffer (s: PByteIOContext; const buf: pchar; size: int);
    cdecl; external dll_name;
  procedure put_le64 (s: PByteIOContext; val: int64);
    cdecl; external dll_name;
  procedure put_be64 (s: PByteIOContext; val: int64);
    cdecl; external dll_name;
  procedure put_le32 (s: PByteIOContext; val: cardinal);
    cdecl; external dll_name;
  procedure put_be32 (s: PByteIOContext; val: cardinal);
    cdecl; external dll_name;
  procedure put_le24 (s: PByteIOContext; val: cardinal);
    cdecl; external dll_name;
  procedure put_be24 (s: PByteIOContext; val: cardinal);
    cdecl; external dll_name;
  procedure put_le16 (s: PByteIOContext; val: cardinal);
    cdecl; external dll_name;
  procedure put_be16 (s: PByteIOContext; val: cardinal);
    cdecl; external dll_name;
  procedure put_tag (s: PByteIOContext; const tag: pchar);
    cdecl; external dll_name;
  procedure put_strz (s: PByteIOContext; const buf: pchar);
    cdecl; external dll_name;

(**
 * fseek() equivalent for ByteIOContext.
 * @return new position or AVERROR.
 *)
  function url_fseek (s: PByteIOContext; offset: offset_t; whence: int): offset_t;
    cdecl; external dll_name;
(**
 * Skip given number of bytes forward.
 * @param offset number of bytes
 *)
  procedure url_fskip (s: PByteIOContext; offset: offset_t);
    cdecl; external dll_name;
(**
 * ftell() equivalent for ByteIOContext.
 * @return position or AVERROR.
 *)
  function url_ftell (s: PByteIOContext): offset_t;
    cdecl; external dll_name;
(**
 * Gets the filesize.
 * @return filesize or AVERROR
 *)
  function url_fsize (s: PByteIOContext): offset_t;
    cdecl; external dll_name;
(**
 * feof() equivalent for ByteIOContext.
 * @return non zero if and only if end of file
 *)
  function url_feof (s: PByteIOContext): int;
    cdecl; external dll_name;
  function url_ferror (s: PByteIOContext): int;
    cdecl; external dll_name;

  function av_url_read_fpause (h: PByteIOContext; pause: int): int;
    cdecl; external dll_name;
  function av_url_read_fseek (h: PByteIOContext; stream_index: int; timestamp: int64; flags: int): offset_t;
    cdecl; external dll_name;

(** @note return URL_EOF (-1) if EOF *)
  function url_fgetc (s: PByteIOContext): int;
    cdecl; external dll_name;

(** @warning currently size is limited
#ifdef __GNUC__
int url_fprintf(ByteIOContext *s, const char *fmt, ...) __attribute__ ((__format__ (__printf__, 2, 3)));
#else
int url_fprintf(ByteIOContext *s, const char *fmt, ...);
#endif
*)

(** @note unlike fgets, the EOL character is not returned and a whole
   line is parsed. return NULL if first char read was EOF *)
  function url_fgets (s: PByteIOContext; buf: pchar; buf_size: int): pchar;
    cdecl; external dll_name;

  procedure put_flush_packet (s: PByteIOContext);
    cdecl; external dll_name;

  function get_buffer (s: PByteIOContext; buf: pchar; size: int): int;
    cdecl; external dll_name;
  function get_partial_buffer (s: PByteIOContext; buf: pchar; size: int): int;
    cdecl; external dll_name;

(** @note return 0 if EOF, so you cannot use it if EOF handling is
   necessary *)
  function get_byte (s: PByteIOContext): int;
    cdecl; external dll_name;
  function get_le24 (s: PByteIOContext): cardinal;
    cdecl; external dll_name;
  function get_le32 (s: PByteIOContext): cardinal;
    cdecl; external dll_name;
  function get_le64 (s: PByteIOContext): int64;
    cdecl; external dll_name;
  function get_le16 (s: PByteIOContext): cardinal;
    cdecl; external dll_name;

  function get_strz (s: PByteIOContext; buf: pchar; maxlen: int): pchar;
    cdecl; external dll_name;
  function get_be16 (s: PByteIOContext): cardinal;
    cdecl; external dll_name;
  function get_be24 (s: PByteIOContext): cardinal;
    cdecl; external dll_name;
  function get_be32 (s: PByteIOContext): cardinal;
    cdecl; external dll_name;
  function get_be64 (s: PByteIOContext): int64;
    cdecl; external dll_name;

  function ff_get_v (bc: PByteIOContext): int64;
    cdecl; external dll_name;

  function url_is_streamed (s: PByteIOContext): int;

(** @note when opened as read/write, the buffers are only used for
   writing *)
  function url_fdopen (var s: PByteIOContext; h: PURLContext): int;
    cdecl; external dll_name;

(** @warning must be called before any I/O *)
  function url_setbufsize (s: PByteIOContext; buf_size: int): int;
    cdecl; external dll_name;
(** Reset the buffer for reading or writing.
 * @note Will drop any data currently in the buffer without transmitting it.
 * @param flags URL_RDONLY to set up the buffer for reading, or URL_WRONLY
 *        to set up the buffer for writing. *)
  function url_resetbuf (s: PByteIOContext; flags: int): int;
    cdecl; external dll_name;

(** @note when opened as read/write, the buffers are only used for
   writing *)
  function url_fopen (var s: PByteIOContext; const filename: pchar; flags: int): int;
    cdecl; external dll_name;
  function url_fclose (s: PByteIOContext): int;
    cdecl; external dll_name;
  function url_fileno (s: PByteIOContext): PURLContext;
    cdecl; external dll_name;

(**
 * Return the maximum packet size associated to packetized buffered file
 * handle. If the file is not packetized (stream like http or file on
 * disk), then 0 is returned.
 *
 * @param s buffered file handle
 * @return maximum packet size in bytes
 *)
  function url_fget_max_packet_size (s: PByteIOContext): int;
    cdecl; external dll_name;

  function url_open_buf (var s: PByteIOContext; buf: PByte; buf_size: int; flags: int): int;
    cdecl; external dll_name;

(** return the written or read size *)
  function url_close_buf (s: PByteIOContext): int;
    cdecl; external dll_name;

(**
 * Open a write only memory stream.
 *
 * @param s new IO context
 * @return zero if no error.
 *)
  function url_open_dyn_buf (var s: PByteIOContext): int;
    cdecl; external dll_name;

(**
 * Open a write only packetized memory stream with a maximum packet
 * size of 'max_packet_size'.  The stream is stored in a memory buffer
 * with a big endian 4 byte header giving the packet size in bytes.
 *
 * @param s new IO context
 * @param max_packet_size maximum packet size (must be > 0)
 * @return zero if no error.
 *)
  function url_open_dyn_packet_buf (var s: PByteIOContext; max_packet_size: int): int;
    cdecl; external dll_name;

(**
 * Return the written size and a pointer to the buffer. The buffer
 *  must be freed with av_free().
 * @param s IO context
 * @param pbuffer pointer to a byte buffer
 * @return the length of the byte buffer
 *)
  function url_close_dyn_buf (s: PByteIOContext; var pbuffer: PByte): int;
    cdecl; external dll_name;

  function ff_crc04C11DB7_update (checksum: cardinal; const buf: PByte; len: cardinal): cardinal;
    cdecl; external dll_name;
  function get_checksum (s: PByteIOContext): cardinal;
    cdecl; external dll_name;
  procedure init_checksum (s: PByteIOContext;
                           update_checksum: pointer; //unsigned long (*update_checksum)(unsigned long c, const uint8_t *p, unsigned int len),
                           checksum: cardinal);
    cdecl; external dll_name;

(* udp.c *)
  function udp_set_remote_url (h: PURLContext; const uri: pchar): int;
    cdecl; external dll_name;
  function udp_get_local_port (h: PURLContext): int;
    cdecl; external dll_name;
  function udp_get_file_handle (h: PURLContext): int;
    cdecl; external dll_name;

implementation

function url_is_streamed (s: PByteIOContext): int;
begin
  Result := s.is_streamed;
end;

end.

