(*
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

 delphi translation @ 25-feb-2008

 *)

(**
 * @file fifo.h
 * A very simple circular buffer FIFO implementation.
 *)

unit fifo;

interface

uses
  windows;

const
  dll_name = 'avutil-49.dll';

type
  int = integer;

  PAVFifoBuffer = ^TAVFifoBuffer;
  TAVFifoBuffer = record
    buffer: PByte;
    rptr, wptr, end_: PByte;
  end;

(**
 * Initializes an AVFifoBuffer.
 * @param *f AVFifoBuffer to initialize
 * @param size of FIFO
 * @return <0 for failure >=0 otherwise
 *)
function av_fifo_init (f: PAVFifoBuffer; size: integer): int;
  cdecl; external dll_name;

(**
 * Frees an AVFifoBuffer.
 * @param *f AVFifoBuffer to free
 *)
procedure av_fifo_free (f: PAVFifoBuffer);
  cdecl; external dll_name;

(**
 * Returns the amount of data in bytes in the AVFifoBuffer, that is the
 * amount of data you can read from it.
 * @param *f AVFifoBuffer to read from
 * @return size
 *)
function av_fifo_size (f: PAVFifoBuffer): int;
  cdecl; external dll_name;

(**
 * Reads data from an AVFifoBuffer.
 * @param *f AVFifoBuffer to read from
 * @param *buf data destination
 * @param buf_size number of bytes to read
 *)
function av_fifo_read (f: PAVFifoBuffer; buf: PByte; buf_size: int): integer;
  cdecl; external dll_name;

(**
 * Feeds data from an AVFifoBuffer to a user supplied callback.
 * @param *f AVFifoBuffer to read from
 * @param buf_size number of bytes to read
 * @param *func generic read function
 * @param *dest data destination
 *)

type
  TFifoFunc = procedure (param1: pointer; param2: pointer; param3: int);

function av_fifo_generic_read (f: PAVFifoBuffer; buf_size: int;
                               func: TFifoFunc;
                               dest: pointer): int;
  cdecl; external dll_name;

(**
 * Writes data into an AVFifoBuffer.
 * @param *f AVFifoBuffer to write to
 * @param *buf data source
 * @param size data size
 *)
procedure av_fifo_write (f: PAVFifoBuffer; const buf: PByte; size: int);
  cdecl; external dll_name;

(**
 * Resizes an AVFifoBuffer.
 * @param *f AVFifoBuffer to resize
 * @param size new AVFifoBuffer size in bytes
 *)
procedure av_fifo_realloc (f: PAVFifoBuffer; size: cardinal);
  cdecl; external dll_name;

(**
 * Reads and discards the specified amount of data from an AVFifoBuffer.
 * @param *f AVFifoBuffer to read from
 * @param size amount of data to read in bytes
 *)
procedure av_fifo_drain (f: PAVFifoBuffer; size: int);
  cdecl; external dll_name;

function av_fifo_peek (f: PAVFifoBuffer; offs: int): byte;

implementation

function av_fifo_peek (f: PAVFifoBuffer; offs: int): byte;
var
  ptr: PByte;
begin
  ptr := pointer (integer (f.rptr) + offs);
  if integer (ptr) >= integer (f.end_) then
    ptr := pointer (integer (ptr) - integer (f.end_) - integer (f.buffer));

  Result := ptr^;
end;


end.
