(*
 * AVOptions
 * copyright (c) 2005 Michael Niedermayer <michaelni@gmx.at>
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

   ===

   revision 15120, 31-aug-2008

   delphi conversion of libavcodec/opt.h

   slightly.fat.hamster@gmail.com, originally by Victor Zinetz

   24-sept-2008

   ===

 *)

unit opt;

interface

uses
  rational;

const
  AV_OPT_FLAG_ENCODING_PARAM  = 1;   ///< a generic parameter which can be set by the user for muxing or encoding
  AV_OPT_FLAG_DECODING_PARAM  = 2;   ///< a generic parameter which can be set by the user for demuxing or decoding
  AV_OPT_FLAG_METADATA        = 4;   ///< some data extracted or inserted into the file like title, comment, ...
  AV_OPT_FLAG_AUDIO_PARAM     = 8;
  AV_OPT_FLAG_VIDEO_PARAM     = 16;
  AV_OPT_FLAG_SUBTITLE_PARAM  = 32;

  dll_name = 'avcodec-51.dll';

type
  int = integer;

  TAVOptionType = (
    FF_OPT_TYPE_FLAGS,
    FF_OPT_TYPE_INT,
    FF_OPT_TYPE_INT64,
    FF_OPT_TYPE_DOUBLE,
    FF_OPT_TYPE_FLOAT,
    FF_OPT_TYPE_STRING,
    FF_OPT_TYPE_RATIONAL,
    FF_OPT_TYPE_BINARY,  ///< offset must point to a pointer immediately followed by an int for the length
    FF_OPT_TYPE_CONST = 128
  );

(**
 * AVOption.
 *)
  PAVOption = ^TAVOption;
  TAVOption = record
    name: pchar;
    (**
     * short English text help.
     * @todo what about other languages
     *)
    help: pchar;
    (**
     * The offset relative to the context structure where the option
     * value is stored. It should be 0 for named constants.
     *)
    offset: int;

    _type: TAVOptionType;

    (**
     * the default value for scalar options
     *)
    default_val: double;
    min: double;                 ///< minimum valid value for the option
    max: double;                 ///< maximum valid value for the option

    flags: int;
//#define AV_OPT_FLAG_ENCODING_PARAM  1   ///< a generic parameter which can be set by the user for muxing or encoding
//#define AV_OPT_FLAG_DECODING_PARAM  2   ///< a generic parameter which can be set by the user for demuxing or decoding
//#define AV_OPT_FLAG_METADATA        4   ///< some data extracted or inserted into the file like title, comment, ...
//#define AV_OPT_FLAG_AUDIO_PARAM     8
//#define AV_OPT_FLAG_VIDEO_PARAM     16
//#define AV_OPT_FLAG_SUBTITLE_PARAM  32
//FIXME think about enc-audio, ... style flags

    (**
     * The logical unit to which the option belongs. Non-constant
     * options and corresponding named constants share the same
     * unit. May be NULL.
     *)
    _unit: pchar;
  end;

(**
 * Looks for an option in \p obj. Looks only for the options which
 * have the flags set as specified in \p mask and \p flags (that is,
 * for which it is the case that opt->flags & mask == flags).
 *
 * @param[in] obj a pointer to a struct whose first element is a
 * pointer to an #AVClass
 * @param[in] name the name of the option to look for
 * @param[in] unit the unit of the option to look for, or any if NULL
 * @return a pointer to the option found, or NULL if no option
 * has been found
 *)
function av_find_opt (obj: pointer; const name: pchar; const _unit: pchar; mask, flags: int): PAVOption;
  cdecl; external dll_name;

(**
 * @see av_set_string2()
 *)
function av_set_string (obj: pointer; const name: pchar; const val: pchar): PAVOption;
  cdecl; external dll_name; deprecated;

(**
 * Sets the field of obj with the given name to value.
 *
 * @param[in] obj A struct whose first element is a pointer to an
 * AVClass.
 * @param[in] name the name of the field to set
 * @param[in] val The value to set. If the field is not of a string
 * type, then the given string is parsed.
 * SI postfixes and some named scalars are supported.
 * If the field is of a numeric type, it has to be a numeric or named
 * scalar. Behavior with more than one scalar and +- infix operators
 * is undefined.
 * If the field is of a flags type, it has to be a sequence of numeric
 * scalars or named flags separated by '+' or '-'. Prefixing a flag
 * with '+' causes it to be set without affecting the other flags;
 * similarly, '-' unsets a flag.
 * @return a pointer to the AVOption corresponding to the field set or
 * NULL if no matching AVOption exists, or if the value \p val is not
 * valid
 * @param alloc when 1 then the old value will be av_freed() and the
 *                     new av_strduped()
 *              when 0 then no av_free() nor av_strdup() will be used
 *)
function av_set_string2 (obj: pointer; const name: pchar; const val: pchar; alloc: int): PAVOption;
  cdecl; external dll_name;

function av_set_double (obj: pointer; const name: pchar; n: double): PAVOption;
  cdecl; external dll_name;
function av_set_q (obj: pointer; const name: pchar; n: TAVRational): PAVOption;
  cdecl; external dll_name;
function av_set_int (obj: pointer; const name: pchar; n: int64): PAVOption;
  cdecl; external dll_name;
function av_get_double (obj: pointer; const name: pchar; var o_out: PAVOption): double;
  cdecl; external dll_name;
function av_get_q (obj: pointer; const name: pchar; var o_out: PAVOption): TAVRational;
  cdecl; external dll_name;
function av_get_int (obj: pointer; const name: pchar; var o_out: PAVOption): int64;
  cdecl; external dll_name;
function av_get_string (obj: pointer; const name: pchar; const o_out: PAVOption; buf: pchar; buf_len: int): pchar;
  cdecl; external dll_name;
function av_next_option (obj: pointer; const last: PAVOption): PAVOption;
  cdecl; external dll_name;
function av_opt_show (obj: pointer; av_log_obj: pointer): int;
  cdecl; external dll_name;
procedure av_opt_set_defaults (s: pointer);
  cdecl; external dll_name;
procedure av_opt_set_defaults2 (s: pointer; mask, flags: int);
  cdecl; external dll_name;

implementation

end.
