(*
 * RTSP definitions
 * Copyright (c) 2002 Fabrice Bellard.
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

 delphi translation @ 25-feb-2008

 *)

unit rtsp;

interface

uses
  windows,
  avformat,
  rtspcodes;

type
  TRTSPProtocol = (RTSP_PROTOCOL_RTP_UDP = 0,
    RTSP_PROTOCOL_RTP_TCP = 1,
    RTSP_PROTOCOL_RTP_UDP_MULTICAST = 2
  );
  int = integer;

const
  dll_name = '';

  RTSP_DEFAULT_PORT   = 554;
  RTSP_MAX_TRANSPORTS = 8;
  RTSP_TCP_MAX_PACKET_SIZE = 1472;
  RTSP_DEFAULT_NB_AUDIO_CHANNELS = 2;
  RTSP_DEFAULT_AUDIO_SAMPLERATE = 44100;
  RTSP_RTP_PORT_MIN = 5000;
  RTSP_RTP_PORT_MAX = 10000;

type
  TRTSPTransportField = record
    interleaved_min, interleaved_max: int;  (**< interleave ids, if TCP transport *)
    port_min, port_max: int; (**< RTP ports *)
    client_port_min, client_port_max: int; (**< RTP ports *)
    server_port_min, server_port_max: int; (**< RTP ports *)
    ttl: int; (**< ttl value *)
    destination: cardinal; (**< destination IP address *)
    protocol: TRTSPProtocol;
  end;

  PRTSPHeader = ^TRTSPHeader; 
  TRTSPHeader = record
    content_length: int;
    status_code: TRTSPStatusCode; (**< response code from server *)
    nb_transports: int;
    (** in AV_TIME_BASE unit, AV_NOPTS_VALUE if not used *)
    range_start, range_end: int64;
    transports: array [0..RTSP_MAX_TRANSPORTS - 1] of TRTSPTransportField ;
    seq: int; (**< sequence number *)
    session_id: array [0..511] of char;
  end;

(** the callback can be used to extend the connection setup/teardown step *)
  TRTSPCallbackAction = (RTSP_ACTION_SERVER_SETUP,
    RTSP_ACTION_SERVER_TEARDOWN,
    RTSP_ACTION_CLIENT_SETUP,
    RTSP_ACTION_CLIENT_TEARDOWN
  );

  TRTSPActionServerSetup = record
    ipaddr: cardinal;
    transport_option: array [0..511] of char;
  end;

{typedef int FFRTSPCallback(enum RTSPCallbackAction action,
                           const char *session_id,
                           char *buf, int buf_size,
                           void *arg);}

  function rtsp_init (): int;
    cdecl; external dll_name;
  procedure rtsp_parse_line (reply: PRTSPHeader; const buf: pchar);
    cdecl; external dll_name;
  function rtsp_pause (s: PAVFormatContext): int;
    cdecl; external dll_name;
  function rtsp_resume (s: PAVFormatContext): int;
    cdecl; external dll_name;

implementation

end.
