(*
 * Copyright (C) 2001-2003 Michael Niedermayer <michaelni@gmx.at>
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

   ===

   revision 27592, 13-sep-2008

   delphi conversion of libavcodec/mem.h

   slightly.fat.hamster@gmail.com, originally by Victor Zinetz

   25-sept-2008

   ===

 *)

unit swscale;

interface

(**
 * @file swscale.h
 * @brief
 *     external api for the swscale stuff
 *)

uses
  windows,
  avutil;

const
  LIBSWSCALE_VERSION_MAJOR = 0;
  LIBSWSCALE_VERSION_MINOR = 6;
  LIBSWSCALE_VERSION_MICRO = 1;

  LIBSWSCALE_VERSION_INT = LIBSWSCALE_VERSION_MAJOR shl 16 +
                           LIBSWSCALE_VERSION_MINOR shl 8 +
                           LIBSWSCALE_VERSION_MICRO;

  SWS_FAST_BILINEAR =    1;
  SWS_BILINEAR      =    2;
  SWS_BICUBIC       =    4;
  SWS_X             =    8;
  SWS_POINT         = $10;
  SWS_AREA          = $20;
  SWS_BICUBLIN      = $40;
  SWS_GAUSS         = $80;
  SWS_SINC          = $100;
  SWS_LANCZOS       = $200;
  SWS_SPLINE        = $400;

  SWS_SRC_V_CHR_DROP_MASK  =   $30000;
  SWS_SRC_V_CHR_DROP_SHIFT =   16;

  SWS_PARAM_DEFAULT        =   123456;

  SWS_PRINT_INFO           =   $1000;

//the following 3 flags are not completely implemented
//internal chrominace subsampling info
  SWS_FULL_CHR_H_INT    = $2000;
//input subsampling info
  SWS_FULL_CHR_H_INP    = $4000;
  SWS_DIRECT_BGR        = $8000;
  SWS_ACCURATE_RND      = $40000;
  SWS_BITEXACT          = $80000;

  SWS_CPU_CAPS_MMX      = $80000000;
  SWS_CPU_CAPS_MMX2     = $20000000;
  SWS_CPU_CAPS_3DNOW    = $40000000;
  SWS_CPU_CAPS_ALTIVEC  = $10000000;
  SWS_CPU_CAPS_BFIN     = $01000000;

  SWS_MAX_REDUCE_CUTOFF: double = 0.002;

  SWS_CS_ITU709         = 1;
  SWS_CS_FCC            = 4;
  SWS_CS_ITU601         = 5;
  SWS_CS_ITU624         = 5;
  SWS_CS_SMPTE170M      = 5;
  SWS_CS_SMPTE240M      = 7;
  SWS_CS_DEFAULT        = 5;

  dll_name = 'swscale-0.dll';

type
  int = integer;
// when used for filters they must have an odd number of elements
// coeffs cannot be shared between vectors
  PSwsVector = ^TSwsVector;
  TSwsVector = record
    coeff: PDouble;
    length: int;
  end;

// vectors can be shared
  PSwsFilter = ^TSwsFilter;
  TSwsFilter = record
    lumH: PSwsVector;
    lumV: PSwsVector;
    chrH: PSwsVector;
    chrV: PSwsVector;
  end;

  PSwsContext = pointer;
  PPByte = ^PByte;

(**
 * Returns the LIBSWSCALE_VERSION_INT constant.
 *)
function swscale_version (): cardinal;
  cdecl; external dll_name;

procedure sws_freeContext (swsContext: PSwsContext);
  cdecl; external dll_name;

function sws_getContext (srcW, srcH: int; srcFormat: TPixelFormat; dstW, dstH: integer; dstFormat: TPixelFormat; flags: int;
                         srcFilter: PSwsFilter; dstFilter: PSwsFilter; param: PDouble): PSwsContext;
  cdecl; external dll_name;

function sws_scale ( context: PSwsContext; src: PByte; srcStride: PInteger; srcSliceY, srcSliceH: int;
                                           dst: PByte; dstStride: PInteger): int;
  cdecl; external dll_name;

function sws_scale_ordered (context: PSwsContext; src: PByte; srcStride: PInteger;
              srcSliceY, srcSliceH: int; dst: PByte; dstStride: PInteger): int;
  deprecated;
  cdecl; external dll_name;

(*
int sws_setColorspaceDetails(struct SwsContext *c, const int inv_table[4], int srcRange, const int table[4], int dstRange, int brightness, int contrast, int saturation);
  cdecl; external dll_name;
int sws_getColorspaceDetails(struct SwsContext *c, int **inv_table, int *srcRange, int **table, int *dstRange, int *brightness, int *contrast, int *saturation);
  cdecl; external dll_name;
SwsVector *sws_getGaussianVec(double variance, double quality);
  cdecl; external dll_name;
SwsVector *sws_getConstVec(double c, int length);
  cdecl; external dll_name;
SwsVector *sws_getIdentityVec(void);
  cdecl; external dll_name;
void sws_scaleVec(SwsVector *a, double scalar);
  cdecl; external dll_name;
void sws_normalizeVec(SwsVector *a, double height);
  cdecl; external dll_name;
void sws_convVec(SwsVector *a, SwsVector *b);
  cdecl; external dll_name;
void sws_addVec(SwsVector *a, SwsVector *b);
  cdecl; external dll_name;
void sws_subVec(SwsVector *a, SwsVector *b);
  cdecl; external dll_name;
void sws_shiftVec(SwsVector *a, int shift);
  cdecl; external dll_name;
SwsVector *sws_cloneVec(SwsVector *a);
  cdecl; external dll_name;

void sws_printVec(SwsVector *a);
  cdecl; external dll_name;
void sws_freeVec(SwsVector *a);
  cdecl; external dll_name;
*)
function sws_getDefaultFilter (lumaGBlur, chromaGBlur, lumaSarpen, chromaSharpen,
                               chromaHShift, chromaVShift: single;
                               verbose: int): PSwsFilter;
  cdecl; external dll_name;
procedure sws_freeFilter (filter: PSwsFilter);
  cdecl; external dll_name;

function sws_getCachedContext (context: PSwsContext;
                               srcW, srcH, srcFormat, dstW, dstH, dstFormat, flags: int;
                               srcFilter: PSwsFilter; dstFilter: PSwsFilter; param: PDouble): PSwsContext;
  cdecl; external dll_name;

implementation

end.
